#!/bin/bash

cmd="$1"
maildir="${2:-/var/qmail/mailnames}"

tmpdir="/root"
badlst="$tmpdir/mboxes_bad.lst"
replace_map="$tmpdir/mboxes_replace.lst"
revert_map="$tmpdir/mboxes_revert.lst"

usage()
{
	echo "USAGE: $0 <action> [maildir]"
	echo ""
	echo "       $0 check   [maildir] - Check mailboxes and gather ones with wrong directories names."
	echo "       $0 encode  [maildir] - Encode wrong directories names the correct form."
	echo "       $0 decode  [maildir] - Decode replaced directories"
	echo ""
	exit 1
}

urlencode() {
    # urlencode <string>
    local length="${#1}"
    for (( i = 0; i < length; i++ )); do
        local c="${1:i:1}"
        case $c in
            [/a-zA-Z0-9.~_-]) printf "$c" ;;
            *) printf '%%%02X' "'$c"
        esac
    done
}

do_check()
{
	echo "Trying to find symbols described in KB article:"
	echo "   https://support.plesk.com/hc/en-us/articles/360006348633-Switch-from-Courier-IMAP-POP3-server-to-Dovecot-via-Plesk-Installer-on-CentOS7-removes-bin-sbin-lib-lib64-symlinks"
	echo ""

	# Save wrong dirs into file
	find $maildir -mindepth 1 -type d -printf "%d_%p\n" | grep -E "\*|\{|\}|\[|\]|\ |\?|\"|'" >$badlst 2>/dev/null

	numbad="`cat $badlst|wc -l`"
	if [ $numbad -eq 0 ]; then
		echo "[OK] There no any mail directories with mentioned symbols."
		echo "     Now you can switch from courier-imap to dovecot IMAP server."
		echo ""
		rm -f $badlst
		exit 0
	fi

	echo "[WARN] There are mail directories with wrong symbols."
	echo "       The wrong directories list has been saved to ${badlst}"

	exit 2
}

do_encode()
{
	echo "Trying to encode wrong directories names.. "
	echo ""

	if [ ! -f "$badlst" ]; then
		echo "Unable to find the directories list($badlist)."
		echo "Please execute '$0 check [mailname]' first."
		echo "
		return 1
	fi

	if [ -f "$replace_map" ]; then
		echo "Previous execution was found."
		echo "Please either revert encoded directories or remove '$replace_map' file."
		echo "
		return 1
	fi

	# Replace dirs
	cat $badlst | sort -r | sed -e 's|[[:digit:]]*_\(.*\)|\1|' >${badlst}.tmp
	while read -r line; do
		path="`dirname "$line"`"
		subdir="`basename "$line"`"
		ensub="`urlencode "$subdir"`"

		echo "$line ==> $path/$ensub" >> $replace_map
		mv -f "$line" "$path/$ensub"

		echo -n "."
	done < ${badlst}.tmp
	rm -f ${badlst}.tmp

	# Save replaced dirs in reverse order to revert changes
	cat $badlst | sort | sed -e 's|[[:digit:]]*_\(.*\)|\1|' >${badlst}.tmp
	while read -r line; do
		path="`dirname "$line"`"
		subdir="`basename "$line"`"
		ensub="`urlencode "$subdir"`"

		echo "$line ==> $path/$ensub" >> $revert_map
	done < ${badlst}.tmp
	rm -f ${badlst}.tmp

	echo ""
	echo "Done! Renamed directories list was saved into '$replace_map' file."
	echo ""

	return 0
}

do_decode()
{
	echo "Trying to revert renamed directories.."
	echo ""

	if [ ! -f "$revert_map" ]; then
		echo "Unable to find the directories list($revert_map)."
		echo "Please execute '$0 encode [mailname]' first."
		echo ""
	       	return 1
	fi

	while read -r line; do
		encoded="`echo "$line" | sed -e 's|.* ==> \(.*\)|\1|'`"
		decoded="`echo "$line" | sed -e 's|\(.*\) ==> .*|\1|'`"

		mv -f "$encoded" "$decoded"
	done < "$revert_map"

	echo "Done! Renamed directories list was saved into '$replace_map' file."
	echo ""

	return 0
}

case "$cmd" in
	check)
		do_check
	;;
	encode)
		do_encode
	;;
	decode)
		do_decode
	;;
	*)
		usage
	;;
esac

exit $?
